// View Synthesis Decoder
// Last Update: Feb, 2014.

// Original code: View Synthesis Reference Software (VSRS) 3.5
//======================================created by Ying Chen =====================================
//===============Tampere University of Technology (TUT)/Nokia Research Center (NRC)===============

#ifdef WIN32
#pragma warning(disable : 4996)
#else
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#endif

#include "ParameterBase.h"
#include "ParameterDec.h"
#include "SubMixOptions.h"

// Modified by NICT. Dec 2011
CParameterEncResiduals::CParameterEncResiduals()
  : m_Mode ( "" )
  , m_uiSourceWidth ( 0 )
  , m_uiSourceHeight ( 0 )
  , m_uiStartFrame ( 0 )
  , m_uiNumberOfFrames ( 0 )

  , m_dNearestDepthValue1 ( 0 )
  , m_dFarthestDepthValue1 ( 0 )
  , m_dNearestDepthValue2 ( 0 )
  , m_dFarthestDepthValue2 ( 0 )
  , m_dNearestDepthValue3 ( 0 )
  , m_dFarthestDepthValue3 ( 0 )

  , m_CameraName1 ( "" )
  , m_CameraName2 ( "" )
  , m_CameraName3 ( "" )
  , m_VertualCameraName1 ( "" )
  , m_VertualCameraName2 ( "" )
  , m_TargetCameraName1 ( "" )
  , m_TargetCameraName2 ( "" )
  , m_TargetCameraName3 ( "" )
  , m_TargetCameraName4 ( "" )
  , m_TargetCameraName6 ( "" )
  , m_TargetCameraName7 ( "" )
  , m_TargetCameraName8 ( "" )
  , m_TargetCameraName9 ( "" )

  , m_CameraParameterFileName ( "" )

  , m_NFTFileName ( "" )

  , m_VName ( "" )
  , m_Bind_DName ( "" )
  , m_Bind_RName ( "" )
  , m_T1Name ( "" )
  , m_T2Name ( "" )
  , m_T3Name ( "" )
  , m_T4Name ( "" )
  , m_T6Name ( "" )
  , m_T7Name ( "" )
  , m_T8Name ( "" )
  , m_T9Name ( "" )
{
}



// Original
CParameterEncResiduals::~CParameterEncResiduals() {
  release();
}



// Small modified by NICT. Dec 2011
Int CParameterEncResiduals::Init( Int argc, Char**  argv ) {
  setup();
  std::string cFilename;

  if ( argc < 2 ) {
    /*
    xPrintUsage();
    release();
    return -1;
    */
    cFilename= "GVDViewSynth.cfg"; //DEFAULT_GVDCONFIG;
  } else {
    cFilename= argv[1] ;
  }

  if ( xReadFromFile( cFilename ) != 1 ) {
    xPrintUsage();
    release();
    return -1;
  }

  if ( xReadFromCommandLine(argc, argv) != 1 ) {
    xPrintUsage();
    release();
    return -1;
  }
  xPrintParam();
  release();

  if ( !xReadCameraParameters() ) return -1;

  return 1;
}



// New impliment by NICT. Dec 2011
void CParameterEncResiduals::xPrintUsage() {
  using namespace std;

  cout << endl << "GVD: ERR: Config file name required." << endl;
  cout <<         "  Config files parameter set:" << endl;

  unsigned int uiParLnCount= 0;
  while ( m_pCfgLines[uiParLnCount] != NULL ) {
    cout << "    " << m_pCfgLines[uiParLnCount]->getTag() << endl;
    uiParLnCount++;
  }
  cout << endl << "      Mode: ""QUARTER / QUATER_L"" or ""QUARTER_ODD / QUARTER_ODD_L / FULL_ODD_L""." << endl;
  cout <<         "        QUARTER / QUATER_L mode is for 2 View setting." << endl;
  cout <<         "        QUARTER_ODD / QUARTER_ODD_L / FULL_ODD_L mode is for 3 View setting." << endl;
  cout <<         "        Without postfix ""_L"" mode is for Half hight/width output." << endl;
  cout <<         "        With postfix ""_L"" mode is for Half hight/width depth map " << endl << endl;
  cout <<         "                                               with full size output." << endl;
  cout <<         "      GVD standard: 2views = QUATER_L, 3views(ATM) = QUARTER_ODD_L, 3views(HTM) = FULL_ODD_L." << endl;
  cout <<         "      File/Position ID 1,2,3.. is LEFT:1 to RIGHT:3." << endl;
}



// Modified by NICT. Dec 2011
unsigned int CParameterEncResiduals::setup() {
  unsigned int uiParLnCount= 0;

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Mode", &m_Mode, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "SourceWidth", &m_uiSourceWidth, 1280 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "SourceHeight", &m_uiSourceHeight, 960 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "TotalNumberOfFrames", &m_uiNumberOfFrames, 1 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "StartFrame", &m_uiStartFrame, 0 );

  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue1", &m_dNearestDepthValue1, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue1", &m_dFarthestDepthValue1, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue2", &m_dNearestDepthValue2, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue2", &m_dFarthestDepthValue2, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue3", &m_dNearestDepthValue3, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue3", &m_dFarthestDepthValue3, 0 );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName1", &m_CameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName2", &m_CameraName2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName3", &m_CameraName3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "VertualCameraName1", &m_VertualCameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "VertualCameraName2", &m_VertualCameraName2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName1", &m_TargetCameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName2", &m_TargetCameraName2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName3", &m_TargetCameraName3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName4", &m_TargetCameraName4, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName6", &m_TargetCameraName6, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName7", &m_TargetCameraName7, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName8", &m_TargetCameraName8, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TargetCameraName9", &m_TargetCameraName9, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraParameterFile", &m_CameraParameterFileName, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NFTFile", &m_NFTFileName, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "View2", &m_VName, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "DepthBind", &m_Bind_DName, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "ResidualsBind", &m_Bind_RName, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target1", &m_T1Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target2", &m_T2Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target3", &m_T3Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target4", &m_T4Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target6", &m_T6Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target7", &m_T7Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target8", &m_T8Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Target9", &m_T9Name, "" );

  m_pCfgLines[uiParLnCount] = NULL;

  return uiParLnCount;
}



// Small modified by NICT. Dec 2011
// CenterDepth: QUATER, QUOTER_L, QUATER_STACK, HALF_QUATER, HALF_QUATER_STACK
// AlignDepth: QUARTER_ODD, QUARTER_ODD_L
bool CParameterEncResiduals::xReadCameraParameters() {
  FILE *f_CPF;
  class NoInfile {};
  try {
    #ifdef WIN32
    f_CPF= _fsopen(m_CameraParameterFileName.c_str(), "rt", _SH_DENYWR); if (f_CPF == NULL) throw NoInfile(); // rb
    #else
    f_CPF= fopen(m_CameraParameterFileName.c_str(), "rt"); if (f_CPF == NULL) throw NoInfile(); // rb
    #endif
  } catch (NoInfile) {
    std::cout << "GVD: ERR: Camera parameter file: Not specify files, or File path/IO error." << std::endl;
  } catch (...) {
    std::cout << "GVD: ERR: Camera parameter file: File IO error." << std::endl;
  }

  bool is3view= ( m_Mode.compare("QUARTER") != 0 && m_Mode.compare("QUARTER_L") != 0 );

  const char* cameraId[CAMERAS];
  try {
    cameraId[0]= m_CameraName1.c_str();
    cameraId[1]= m_CameraName2.c_str();
    cameraId[2]= "";
    cameraId[3]= m_VertualCameraName1.c_str();
    cameraId[4]= "";
    cameraId[5]= m_TargetCameraName1.c_str();
    cameraId[6]= m_TargetCameraName2.c_str();
    cameraId[7]= m_TargetCameraName3.c_str();
    cameraId[8]= m_TargetCameraName4.c_str();
    cameraId[9]= "";
    cameraId[10]= "";
    cameraId[11]= "";
    cameraId[12]= "";
    if ( is3view ) {
      cameraId[2]= m_CameraName3.c_str();
      cameraId[4]= m_VertualCameraName2.c_str();
      cameraId[9]= m_TargetCameraName6.c_str();
      cameraId[10]= m_TargetCameraName7.c_str();
      cameraId[11]= m_TargetCameraName8.c_str();
      cameraId[12]= m_TargetCameraName9.c_str();
    }
  } catch (...) {
    std::cout << "GVD: ERR: Camera parameter file: File IO error." << std::endl;
  }

#if ATM_CAMERAPARAM

  char line[256]
  Double viewID[3], startF[3] endF[3], FocalLength[3], Position[3], cShift[3], zNear[3], zFar[3];
  if ( fseek( f_CPF, 0, SEEK_SET ) ) return false;
  if ( fgets( line, 256, f_CPF ) ) return false; // 読み捨て
  for (unsigned int i= 0; i < 2; i++) { // 2View、3View 同じ
    fscanf(f_CPF, "%8d  %10d  %10d  %-15.5f  %-15.5f %-15.5f  %-15.5f  %-15.5f\n", viewID[i], startF[i], endF[i], FocalLength[i], Position[i], cShift[i], zNear[i], zFar[i] );
  }
  // この版では、Start/End Frame と CShift は無視する。


#else

  char id[255];
  double gomi[2];
  bool found;
  int read= 0;
  for (unsigned int i= 0; i < CAMERAS; i++) {
    if (  !is3view && (i == 2 || i == 4 || i >= 9) ) continue;
    if ( ( m_Mode.compare("QUARTER_ODD") || m_Mode.compare("QUARTER_ODD_L") || m_Mode.compare("FULL_ODD_L") ) && (i == 3 || i == 4) ) continue;
    if ( fseek(f_CPF, 0, SEEK_SET) ) return false;

    found= false;
    while( fscanf(f_CPF, "%s", id) == 1 ) {
      read= 0;
      if( strcmp(cameraId[i], id) == 0 ) {
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[0][0], &m_camParam[i].m_fIntrinsicMatrix[0][1], &m_camParam[i].m_fIntrinsicMatrix[0][2]);
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[1][0], &m_camParam[i].m_fIntrinsicMatrix[1][1], &m_camParam[i].m_fIntrinsicMatrix[1][2]);
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[2][0], &m_camParam[i].m_fIntrinsicMatrix[2][1], &m_camParam[i].m_fIntrinsicMatrix[2][2]);
        read += fscanf(f_CPF, "%lf %lf", &gomi[0], &gomi[1]);
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[0][0], &m_camParam[i].m_fExtrinsicMatrix[0][1], &m_camParam[i].m_fExtrinsicMatrix[0][2], &m_camParam[i].m_fTranslationVector[0] );
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[1][0], &m_camParam[i].m_fExtrinsicMatrix[1][1], &m_camParam[i].m_fExtrinsicMatrix[1][2], &m_camParam[i].m_fTranslationVector[1] );
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[2][0], &m_camParam[i].m_fExtrinsicMatrix[2][1], &m_camParam[i].m_fExtrinsicMatrix[2][2], &m_camParam[i].m_fTranslationVector[2] );
        if(read!=23) return false;
        found= true;
        break;
      }
    }

    if (!found) {
      std::cout << "GVD: ERR: Camera ID, not found in the camera parameter file: " << cameraId[i] << std::endl;
      printf("CameraID: %s\n", cameraId[i]);
      return false;
    }
  }

#endif

  return true;
}

