#pragma once
// View Synthesis Encoder for Depth
// Dev Enviroment:
//    Windows 7 64bit with Visual Studio 2010 C++ x64
//    OpenCV 2.4.2 64bit
// Last Update: Jan, 2012.

// Original code: View Synthesis Reference Software (VSRS) 3.5
//======================================created by Ying Chen =====================================
//===============Tampere University of Technology (TUT)/Nokia Research Center (NRC)===============

// # Any ID: LEFT to Right: 1, 2, 3

#include "iostream"
#include "string"

#include "ParameterBase.h"
#include "CameraParameter.h"

#define CAMERAS 5


class CParameterEncDepth : public ParameterBase {
public:
  CParameterEncDepth();
  virtual ~CParameterEncDepth();

  Int Init( Int argc, Char** argv );
  
  void setMode( std::string s ) { m_Mode= s; }
  void setSourceWidth( unsigned int ui ) { m_uiSourceWidth= ui; }
  void setSourceHeight( unsigned int ui ) { m_uiSourceHeight= ui; }
  void setNumberOfFrames( unsigned int ui ) { m_uiNumberOfFrames= ui; }
  void setStartFrame( unsigned int ui ) { m_uiStartFrame= ui; }

  void setNearestDepthValue1( double d ) { m_dNearestDepthValue1= d; }
  void setFarthestDepthValue1( double d ) { m_dFarthestDepthValue1= d; }
  void setNearestDepthValue2( double d ) { m_dNearestDepthValue2= d; }
  void setFarthestDepthValue2( double d ) { m_dFarthestDepthValue2= d; }
  void setNearestDepthValue3( double d ) { m_dNearestDepthValue3= d; }
  void setFarthestDepthValue3( double d ) { m_dFarthestDepthValue3= d; }

  void setCameraParameterFileName( std::string s ) { m_CameraParameterFileName= s; };

  void setNearFN1( std::string s ) { m_NearFN1= s; };
  void setFarFN1( std::string s ) { m_FarFN1= s; };
  void setTransFN1( std::string s ) { m_TransFN1= s; };
  void setNearFN2( std::string s ) { m_NearFN2= s; };
  void setFarFN2( std::string s ) { m_FarFN2= s; };
  void setTransFN2( std::string s ) { m_TransFN2= s; };
  void setNearFN3( std::string s ) { m_NearFN3= s; };
  void setFarFN3( std::string s ) { m_FarFN3= s; };
  void setTransFN3( std::string s ) { m_TransFN3= s; };
  void setNearFNV1( std::string s ) { m_NearFNV1= s; };
  void setFarFNV1( std::string s ) { m_FarFNV1= s; };
  void setTransFNV1( std::string s ) { m_TransFNV1= s; };
  void setNearFNV2( std::string s ) { m_NearFNV2= s; };
  void setFarFNV2( std::string s ) { m_FarFNV2= s; };
  void setTransFNV2( std::string s ) { m_TransFNV2= s; };

  void set_D1Name( std::string s ) { m_D1Name= s; };
  void set_D2Name( std::string s ) { m_D2Name= s; };
  void set_D3Name( std::string s ) { m_D3Name= s; };
  void setBind_DName( std::string s ) { m_Bind_DName= s; };
  void setBind_DDName( std::string s ) { m_Bind_DDName= s; }; // Apr16 For Dummy Depth

  const std::string getMode() { return m_Mode; }
  const unsigned int getSourceWidth() { return m_uiSourceWidth; }
  const unsigned int getSourceHeight() { return m_uiSourceHeight; }
  const unsigned int getNumberOfFrames() { return m_uiNumberOfFrames; }
  const unsigned int getStartFrame() { return m_uiStartFrame; }

  const double getNearestDepthValue1() { return m_dNearestDepthValue1; }
  const double getFarthestDepthValue1() { return m_dFarthestDepthValue1; }
  const double getNearestDepthValue2() { return m_dNearestDepthValue2; }
  const double getFarthestDepthValue2() { return m_dFarthestDepthValue2; }
  const double getNearestDepthValue3() { return m_dNearestDepthValue3; }
  const double getFarthestDepthValue3() { return m_dFarthestDepthValue3; }

  const std::string getCameraParameterFileName() { return m_CameraParameterFileName; }

  const std::string getNearFN1() { return m_NearFN1; }
  const std::string getFarFN1() { return m_FarFN1; }
  const std::string getTransFN1() { return m_TransFN1; }
  const std::string getNearFN2() { return m_NearFN2; }
  const std::string getFarFN2() { return m_FarFN2; }
  const std::string getTransFN2() { return m_TransFN2; }
  const std::string getNearFN3() { return m_NearFN3; }
  const std::string getFarFN3() { return m_FarFN3; }
  const std::string getTransFN3() { return m_TransFN3; }
  const std::string getNearFNV1() { return m_NearFNV1; }
  const std::string getFarFNV1() { return m_FarFNV1; }
  const std::string getTransFNV1() { return m_TransFNV1; }
  const std::string getNearFNV2() { return m_NearFNV2; }
  const std::string getFarFNV2() { return m_FarFNV2; }
  const std::string getTransFNV2() { return m_TransFNV2; }
  
  const std::string getCameraName1() { return m_CameraName1; }
  const std::string getCameraName2() { return m_CameraName2; }
  const std::string getCameraName3() { return m_CameraName3; }
  const std::string getVertualCameraName1() { return m_VertualCameraName1; }
  const std::string getVertualCameraName2() { return m_VertualCameraName2; }

  const std::string get_D1Name() { return m_D1Name; }
  const std::string get_D2Name() { return m_D2Name; }
  const std::string get_D3Name() { return m_D3Name; }
  const std::string getBind_DName() { return m_Bind_DName; }
  const std::string getBind_DDName() { return m_Bind_DDName; } // Apr16 For Dummy Depth

  void xPrintUsage();

  Double *getMat_Ex_1() {return &m_camParam[0].m_fExtrinsicMatrix[0][0];}
  Double *getMat_Ex_2() {return &m_camParam[1].m_fExtrinsicMatrix[0][0];}
  Double *getMat_Ex_3() {return &m_camParam[2].m_fExtrinsicMatrix[0][0];}
  Double *getMat_Ex_V1() {return &m_camParam[3].m_fExtrinsicMatrix[0][0];}
  Double *getMat_Ex_V2() {return &m_camParam[4].m_fExtrinsicMatrix[0][0];}

  Double *getMat_In_1() {return &m_camParam[0].m_fIntrinsicMatrix[0][0];}
  Double *getMat_In_2() {return &m_camParam[1].m_fIntrinsicMatrix[0][0];}
  Double *getMat_In_3() {return &m_camParam[2].m_fIntrinsicMatrix[0][0];}
  Double *getMat_In_V1() {return &m_camParam[3].m_fIntrinsicMatrix[0][0];}
  Double *getMat_In_V2() {return &m_camParam[4].m_fIntrinsicMatrix[0][0];}

  Double *getMat_Trans_1() {return &m_camParam[0].m_fTranslationVector[0];}
  Double *getMat_Trans_2() {return &m_camParam[1].m_fTranslationVector[0];}
  Double *getMat_Trans_3() {return &m_camParam[2].m_fTranslationVector[0];}
  Double *getMat_Trans_V1() {return &m_camParam[3].m_fTranslationVector[0];}
  Double *getMat_Trans_V2() {return &m_camParam[4].m_fTranslationVector[0];}


protected:
  unsigned int setup();


private:
  std::string m_Mode;
  unsigned int m_uiSourceWidth;
  unsigned int m_uiSourceHeight;
  unsigned int m_uiNumberOfFrames;
  unsigned int m_uiStartFrame; 
  
  double m_dNearestDepthValue1;
  double m_dFarthestDepthValue1;
  double m_dNearestDepthValue2;
  double m_dFarthestDepthValue2;
  double m_dNearestDepthValue3;
  double m_dFarthestDepthValue3;

  std::string m_CameraName1;
  std::string m_CameraName2;
  std::string m_CameraName3;
  std::string m_VertualCameraName1;
  std::string m_VertualCameraName2;

  std::string m_CameraParameterFileName;
  
  std::string m_NearFN1;
  std::string m_FarFN1;
  std::string m_TransFN1;
  std::string m_NearFN2;
  std::string m_FarFN2;
  std::string m_TransFN2;
  std::string m_NearFN3;
  std::string m_FarFN3;
  std::string m_TransFN3;
  std::string m_NearFNV1;
  std::string m_FarFNV1;
  std::string m_TransFNV1;
  std::string m_NearFNV2;
  std::string m_FarFNV2;
  std::string m_TransFNV2;
  
  std::string m_D1Name;
  std::string m_D2Name;
  std::string m_D3Name;
  std::string m_Bind_DName;
  std::string m_Bind_DDName; // Apr16 For Dummy Depth

  // 0: Camera1, 1: Camera2, 2: Camera3, 3: VertualCamera1, 4:VertualCamera2
  CCameraParameters m_camParam[CAMERAS];

  bool xReadCameraParameters();
};
