// View Synthesis Encoder for Depth
// Dev Enviroment:
//    Windows 7 64bit with Visual Studio 2010 C++ x64
//    OpenCV 2.4.2 64bit
// Last Update: Feb, 2014.

// Original code: View Synthesis Reference Software (VSRS) 3.5
//======================================created by Ying Chen =====================================
//===============Tampere University of Technology (TUT)/Nokia Research Center (NRC)===============

#ifdef WIN32
#pragma warning(disable : 4996)
#else
#include <stdio.h>
#include <string.h>
#endif

#include "ParameterBase.h"
#include "ParameterEncDepth.h"
#include "SubMixOptions.h"

// Modified by NICT. Dec 2011
CParameterEncDepth::CParameterEncDepth()
  : m_Mode ( "" )
  , m_uiSourceWidth ( 0 )
  , m_uiSourceHeight ( 0 )
  , m_uiStartFrame ( 0 ) 
  , m_uiNumberOfFrames ( 0 )

  , m_dNearestDepthValue1 ( 0 )
  , m_dFarthestDepthValue1 ( 0 )
  , m_dNearestDepthValue2 ( 0 )
  , m_dFarthestDepthValue2 ( 0 )
  , m_dNearestDepthValue3 ( 0 )
  , m_dFarthestDepthValue3 ( 0 )

  , m_CameraName1 ( "" )
  , m_CameraName2 ( "" )
  , m_CameraName3 ( "" )
  , m_VertualCameraName1 ( "" )
  , m_VertualCameraName2 ( "" )

  , m_CameraParameterFileName ( "" )

  , m_NearFN1 ( "" )
  , m_FarFN1 ( "" )
  , m_TransFN1 ( "" )
  , m_NearFN2 ( "" )
  , m_FarFN2 ( "" )
  , m_TransFN2 ( "" )
  , m_NearFN3 ( "" )
  , m_FarFN3 ( "" )
  , m_TransFN3 ( "" )
  , m_NearFNV1 ( "" )
  , m_FarFNV1 ( "" )
  , m_TransFNV1 ( "" )
  , m_NearFNV2 ( "" )
  , m_FarFNV2 ( "" )
  , m_TransFNV2 ( "" )

  , m_D1Name ( "" )
  , m_D2Name ( "" )
  , m_D3Name ( "" )
  , m_Bind_DName ( "" )
  , m_Bind_DDName ( "" ) // Apr16 For Dummy Depth
{
}



// Original
CParameterEncDepth::~CParameterEncDepth() {
  release();
}



// Small modified by NICT. Dec 2011
Int CParameterEncDepth::Init( Int argc, Char**  argv ) {
  setup();

  if ( argc < 2 ) {
    xPrintUsage();
    release();
    return -1;
  }
  
  std::string cFilename= argv[1] ; 
  if ( xReadFromFile( cFilename ) != 1 ) {
    xPrintUsage();
    release();
    return -1;
  }

  if ( xReadFromCommandLine(argc, argv) != 1 ) {
    xPrintUsage();
    release();
    return -1;
  }
  xPrintParam();
  release();

  if ( !xReadCameraParameters() ) return -1;

  return 1;  
}



// New impliment by NICT. Dec 2011
void CParameterEncDepth::xPrintUsage() {
  using namespace std;
  
  cout << endl << "VSencD: ERR: Config file name required." << endl;
  cout <<         "  Config files parameter set:" << endl;

  unsigned int uiParLnCount= 0;
  while ( m_pCfgLines[uiParLnCount] != NULL ) {
    cout << "    " << m_pCfgLines[uiParLnCount]->getTag() << endl;
    uiParLnCount++;
  }
  cout << endl << "      Mode: ""QUARTER"" or ""QUARTER_ODD / QUARTER_STACK / HALF_QUARTER_STACK""." << endl;
  cout <<         "      QUARTER mode is for 2 View setting." << endl;
  cout <<         "      FULL_ODD / QUARTER_ODD / QUARTER_STACK / HALF_QUARTER_STACK mode is for 3 View setting." << endl;
  cout <<         "      File/Position ID 1,2,3 is LEFT:1 to RIGHT:3." << endl;
}



// Modified by NICT. Dec 2011
unsigned int CParameterEncDepth::setup() {
  unsigned int uiParLnCount= 0;

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Mode", &m_Mode, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "SourceWidth", &m_uiSourceWidth, 1280 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "SourceHeight", &m_uiSourceHeight, 960 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "TotalNumberOfFrames", &m_uiNumberOfFrames, 1 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "StartFrame", &m_uiStartFrame, 0 );

  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue1", &m_dNearestDepthValue1, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue1", &m_dFarthestDepthValue1, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue2", &m_dNearestDepthValue2, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue2", &m_dFarthestDepthValue2, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue3", &m_dNearestDepthValue3, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue3", &m_dFarthestDepthValue3, 0 );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName1", &m_CameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName2", &m_CameraName2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName3", &m_CameraName3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "VertualCameraName1", &m_VertualCameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "VertualCameraName2", &m_VertualCameraName2, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraParameterFile", &m_CameraParameterFileName, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Near1", &m_NearFN1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Far1", &m_FarFN1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Trans1", &m_TransFN1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Near2", &m_NearFN2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Far2", &m_FarFN2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Trans2", &m_TransFN2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Near3", &m_NearFN3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Far3", &m_FarFN3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Trans3", &m_TransFN3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearV1", &m_NearFNV1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarV1", &m_FarFNV1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransV1", &m_TransFNV1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearV2", &m_NearFNV2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarV2", &m_FarFNV2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransV2", &m_TransFNV2, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Depth1", &m_D1Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Depth2", &m_D2Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Depth3", &m_D3Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "DepthBind", &m_Bind_DName, "" );

  m_pCfgLines[uiParLnCount] = NULL;

  return uiParLnCount;
}



// Small modified by NICT. Dec 2011
bool CParameterEncDepth::xReadCameraParameters() {
  FILE *f_CPF;
  class NoInfile {};
  try {
    #ifdef WIN32
    f_CPF= _fsopen(m_CameraParameterFileName.c_str(), "rt", _SH_DENYWR); if (f_CPF == NULL) throw NoInfile(); // rb
    #else
    f_CPF= fopen(m_CameraParameterFileName.c_str(), "rb"); if (f_CPF == NULL) throw NoInfile();
    #endif
  } catch (NoInfile) { 
    std::cout << "VSencD: ERR: Camera parameter file: Not specify files, or File path/IO error." << std::endl;
    return false;
  } catch (...) {
    std::cout << "VSencD: ERR: Camera parameter file: File IO error." << std::endl;
    return false;
  }
  
  unsigned int mode= QUARTER_STACK;
  if ( m_Mode.compare("QUARTER") == 0 ) mode= QUARTER;
  if ( m_Mode.compare("QUARTER_ODD") == 0 ) mode= QUARTER_ODD;
  if ( m_Mode.compare("FULL_ODD") == 0 ) mode= FULL_ODD;

  const char* cameraId[CAMERAS];
  try {
    cameraId[0]= m_CameraName1.c_str();
    cameraId[1]= m_CameraName2.c_str();
    cameraId[2]= "";
    cameraId[3]= m_VertualCameraName1.c_str();
    cameraId[4]= "";
    if ( mode != QUARTER ) {
      cameraId[2] = m_CameraName3.c_str();
      if ( mode != QUARTER_ODD && mode != FULL_ODD ) cameraId[4] = m_VertualCameraName2.c_str();
    }
  } catch (...) {
    std::cout << "VSencD: ERR: Camera parameter file: File IO error." << std::endl;
    return false;
  }

  char id[255];
  double gomi[2];
  bool found;
  int read= 0;
  for (unsigned int i= 0; i < CAMERAS; i++) {
    if (  mode == QUARTER && (i == 2 || i == 4) ) continue; 
    if (  mode == QUARTER_ODD && i == 4 ) continue; 
    if (  mode == FULL_ODD && i == 4 ) continue; 
    if ( fseek(f_CPF, 0, SEEK_SET) ) return 0;

    found= false;
    while( fscanf(f_CPF, "%s", id) == 1 ) {
      read= 0;
      if( strcmp(cameraId[i], id) == 0 ) {
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[0][0], &m_camParam[i].m_fIntrinsicMatrix[0][1], &m_camParam[i].m_fIntrinsicMatrix[0][2]);
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[1][0], &m_camParam[i].m_fIntrinsicMatrix[1][1], &m_camParam[i].m_fIntrinsicMatrix[1][2]);
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[2][0], &m_camParam[i].m_fIntrinsicMatrix[2][1], &m_camParam[i].m_fIntrinsicMatrix[2][2]);
        read += fscanf(f_CPF, "%lf %lf", &gomi[0], &gomi[1]);
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[0][0], &m_camParam[i].m_fExtrinsicMatrix[0][1], &m_camParam[i].m_fExtrinsicMatrix[0][2], &m_camParam[i].m_fTranslationVector[0] );
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[1][0], &m_camParam[i].m_fExtrinsicMatrix[1][1], &m_camParam[i].m_fExtrinsicMatrix[1][2], &m_camParam[i].m_fTranslationVector[1] );
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[2][0], &m_camParam[i].m_fExtrinsicMatrix[2][1], &m_camParam[i].m_fExtrinsicMatrix[2][2], &m_camParam[i].m_fTranslationVector[2] );
        if(read!=23) return false;
        found= true;
      }
    }
    
    if (!found) {
      std::cout << "VSencD: ERR: Camera ID, not found in the camera parameter file: " << cameraId[i] << std::endl;
      return false;
    }
  }

  return true;
}

