#pragma once
// View Synthesis Encoder for Residuals
// Dev Enviroment:
//    Windows 7 64bit with Visual Studio 2010 C++ x64
//    OpenCV 2.4.5 64bit
// Original code: View Synthesis Reference Software (VSRS) 3.5
// Last Update: Jul, 2013.
#include <utility>
#include <string>
#include "SubMixOptions.h"
#include "SubSepMix.h"
#include "ReturnCode.h" 
#include "ParameterEncResidual.h"
#include "ResidualSynthesis.h"

#include "DEBUGOUT.h"

namespace view_synthesis {
  namespace residuals_subsep_options {
    using namespace std;

    CParameterEncResiduals cParameter;

    unsigned int mode;
    unsigned int numView;
    typedef pair<unsigned int, string> modepair;
    modepair mode1(QUARTER, "QUARTER"); // VIEW_2
    modepair mode11(QUARTER_L, "QUARTER_L"); // for VIEW_2 with Full hight/width.
    // HALF_QUARTER, OUT                    VIEW_2
    modepair mode2(QUARTER_STACK, "QUARTER_STACK");
    modepair mode3(HALF_QUARTER_STACK, "HALF_QUARTER_STACK");
    modepair mode4(QUARTER_ODD, "QUARTER_ODD");
    modepair mode5(QUARTER_ODD_L, "QUARTER_ODD_L");
    modepair mode6(FULL_ODD_L, "FULL_ODD_L");

    unsigned int checkOption(string option) {
      if ( 0 == option.compare(mode1.second) ) {mode= mode1.first; numView= VIEW_2; return mode;}
      if ( 0 == option.compare(mode11.second) ) {mode= mode11.first; numView= VIEW_2; return mode;}
      if ( 0 == option.compare(mode2.second) ) {mode= mode2.first; numView= VIEW_3; return mode;}
      if ( 0 == option.compare(mode3.second) ) {mode= mode3.first; numView= VIEW_3; return mode;}
      if ( 0 == option.compare(mode4.second) ) {mode= mode4.first; numView= VIEW_3; return mode;}
      if ( 0 == option.compare(mode5.second) ) {mode= mode5.first; numView= VIEW_3; return mode;}
      if ( 0 == option.compare(mode6.second) ) {mode= mode6.first; numView= VIEW_3; return mode;}
      cout << "GVD: ERR: Config parameter: Mode: ->" << option << "<- Unknown." << endl << endl;
      return UNKNOWN;
    }

    bool Offset= false;
    typedef pair<bool, string> foffsetpair;
    foffsetpair T(true, "YES");
    foffsetpair F(false, "NO");
    foffsetpair D(false, "");

    unsigned int checkOffset(string option) {
      if ( 0 == option.compare(T.second) ) {Offset= T.first; return 0;}
      if ( 0 == option.compare(F.second) ) {Offset= F.first; return 0;}
      if ( 0 == option.compare(D.second) ) {Offset= D.first; return 0;}
      cout << "GVD: ERR: Config parameter: Offset: ->" << option << "<- Unknown." << endl << endl;
      return UNKNOWN;
    }
    

    bool VariParam= false;
    unsigned int width, height, half_width, half_height;
    unsigned int start, total;
    double near1, far1, trans1, near2, far2, trans2, near3, far3, trans3;
    double nearV1, farV1, transV1, nearV2, farV2, transV2;
    double nearA1, farA1, transA1, nearA2, farA2, transA2, nearA3, farA3, transA3, nearA4, farA4, transA4;
    class NoEven {};
    class BadSeq {};
    class MixPosNeg {};

    int initParam() {
      try {
        width= cParameter.getSourceWidth();
        height= cParameter.getSourceHeight();
        half_width= width / 2;
        half_height= height / 2;
        if ( width % 2 != 0 || height % 2 != 0) throw NoEven();
      } catch (NoEven) {
        cout << "GVD: ERR: Config parameter: ""SourceWidth"" or ""SourceHeight"" Odd, No even." << endl << endl;
        return BAD_CONFIG_OPTIONS;
      } catch (...) {
        cout << "GVD: ERR: Config parameter: ""SourceWidth"" or ""SourceHeight""." << endl << endl;
        return BAD_CONFIG_OPTIONS;
      }

      try {
        start= cParameter.getStartFrame();
        total= cParameter.getNumberOfFrames();
        // if ( start >= total ) throw BadSeq();
      } catch (BadSeq) {
        cout << "GVD: ERR: Config parameter: ""StartFrame"", ""NumberOfFrames""  Bad order." << endl << endl;
        return BAD_CONFIG_OPTIONS;
      } catch (...) {
        cout << "GVD: ERR: Config parameter: ""StartFrame"" or ""NumberOfFrames""." << endl << endl;
        return BAD_CONFIG_OPTIONS;
      }
      
      if ( cParameter.getNearestDepthValue1() == 0 && cParameter.getFarthestDepthValue1() == 0 ) 
        VariParam= true;
      else
        try {
          near1= cParameter.getNearestDepthValue1();
          far1= cParameter.getFarthestDepthValue1();
          near2= cParameter.getNearestDepthValue2();
          far2= cParameter.getFarthestDepthValue2();
          if ( numView == VIEW_2 ) { // 2 View ////////////////////////////////////////////////////////////////////
            near3= 0;
            far3= 0;
            if ( near1<0 && (far1>0 || near2>0 || far2>0) ) throw MixPosNeg();
            if ( near1>0 && (far1<0 || near2<0 || far2<0) ) throw MixPosNeg();
            if ( (near1==0 && far1==0) || (near2==0 && far2==0) ) throw MixPosNeg();
            if ( near1==far1 || near2==far2 ) throw MixPosNeg();
          } else { //  VIEW_3 //////////////////////////////////////////////////////////////////
            near3= cParameter.getNearestDepthValue3();
            far3= cParameter.getFarthestDepthValue3();
            if ( near1<0 && (far1>0 || near2>0 || far2>0 || near3>0 || far3>0) ) throw MixPosNeg();
            if ( near1>0 && (far1<0 || near2<0 || far2<0 || near3<0 || far3<0) ) throw MixPosNeg();
            if ( (near1==0 && far1==0) || (near2==0 && far2==0)  || (near3==0 && far3==0) ) throw MixPosNeg();
            if ( near1==far1 || near2==far2 || near3==far3 ) throw MixPosNeg();
          }
        } catch (MixPosNeg){
          cout << "GVD: ERR: Config parameter: ""NearestDepthValue"" and ""FarthestDepthValue""  Mixed positive and negative." << endl << endl;
          return BAD_CONFIG_OPTIONS;
        } catch (...) {
          cout << "GVD: ERR: Config parameter: ""NearestDepthValue"" or ""FarthestDepthValue""." << endl << endl;
          return BAD_CONFIG_OPTIONS;
        }
    
      return NORMAL_END;
    }


    // QUARTER_STACK / HALF_QUARTER_STACK
    //   View: [LEFT]=V1, [RIGHT]=V2, [Depth]=BindD -> Stack R1(upper) with R3(bottom)
    // QUARTER
    //   View: [LEFT]=V1, [Depth]=BindD(Alone) -> R1.5 (Alone, not binded!)
    FILE *f_V1, *f_V2, *f_V3, *f_BindD, *f_BindR;
    #ifdef WIN32
    #ifdef DEBUGOUT
    FILE *f_Out1, *f_Out2;
    errno_t err_Out1, err_Out2;
    #endif
    errno_t err_BindR;
    #endif
    
    bool workR= true;
    class NoInfile {};
    class NoOutfile {};
    // for VariParam
    FILE *f_D1near, *f_D1far, *f_D1trans;
    FILE *f_D2near, *f_D2far, *f_D2trans;
    FILE *f_D3near, *f_D3far, *f_D3trans;
    FILE *f_V1near, *f_V1far, *f_V1trans;
    FILE *f_V3near, *f_V3far, *f_V3trans;
    FILE *f_A1near, *f_A1far, *f_A1trans;
    FILE *f_A2near, *f_A2far, *f_A2trans;
    FILE *f_A3near, *f_A3far, *f_A3trans;
    FILE *f_A4near, *f_A4far, *f_A4trans;

    int initFile() {
      try {
        #ifdef WIN32
        f_V1= _fsopen(cParameter.get_V1Name().c_str(), "rb", _SH_DENYWR); if (f_V1 == NULL) throw NoInfile();
        f_V2= _fsopen(cParameter.get_V2Name().c_str(), "rb", _SH_DENYWR); if (f_V2 == NULL) throw NoInfile();
        f_BindD= _fsopen(cParameter.getBind_DName().c_str(), "rb", _SH_DENYWR); if (f_BindD == NULL) throw NoInfile();
        err_BindR= fopen_s(&f_BindR, cParameter.getBind_RName().c_str(), "wb"); if (err_BindR != 0) throw NoOutfile();
        if ( VariParam ) {
          f_D1near= _fsopen(cParameter.getNearFN1().c_str(), "rb", _SH_DENYWR); if (f_D1near == NULL) throw NoInfile();
          f_D1far= _fsopen(cParameter.getFarFN1().c_str(), "rb", _SH_DENYWR); if (f_D1far == NULL) throw NoInfile();
          f_D1trans= _fsopen(cParameter.getTransFN1().c_str(), "rb", _SH_DENYWR); if (f_D1trans == NULL) throw NoInfile();
          f_D2near= _fsopen(cParameter.getNearFN2().c_str(), "rb", _SH_DENYWR); if (f_D2near == NULL) throw NoInfile();
          f_D2far= _fsopen(cParameter.getFarFN2().c_str(), "rb", _SH_DENYWR); if (f_D2far == NULL) throw NoInfile();
          f_D2trans= _fsopen(cParameter.getTransFN2().c_str(), "rb", _SH_DENYWR); if (f_D2trans == NULL) throw NoInfile();
          f_V1near= _fsopen(cParameter.getNearFNV1().c_str(), "rb", _SH_DENYWR); if (f_V1near == NULL) throw NoInfile();
          f_V1far= _fsopen(cParameter.getFarFNV1().c_str(), "rb", _SH_DENYWR); if (f_V1far == NULL) throw NoInfile();
          f_V1trans= _fsopen(cParameter.getTransFNV1().c_str(), "rb", _SH_DENYWR); if (f_V1trans == NULL) throw NoInfile();
          f_A1near= _fsopen(cParameter.getNearFNA1().c_str(), "rb", _SH_DENYWR); if (f_A1near == NULL) throw NoInfile();
          f_A1far= _fsopen(cParameter.getFarFNA1().c_str(), "rb", _SH_DENYWR); if (f_A1far == NULL) throw NoInfile();
          f_A1trans= _fsopen(cParameter.getTransFNA1().c_str(), "rb", _SH_DENYWR); if (f_A1trans == NULL) throw NoInfile();
          f_A2near= _fsopen(cParameter.getNearFNA2().c_str(), "rb", _SH_DENYWR); if (f_A2near == NULL) throw NoInfile();
          f_A2far= _fsopen(cParameter.getFarFNA2().c_str(), "rb", _SH_DENYWR); if (f_A2far == NULL) throw NoInfile();
          f_A2trans= _fsopen(cParameter.getTransFNA2().c_str(), "rb", _SH_DENYWR); if (f_A2trans == NULL) throw NoInfile();
          if ( fseek(f_D1near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D1far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D1trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D2near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D2far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D2trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_V1near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_V1far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_V1trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A1near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A1far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A1trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A2near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A2far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A2trans, 0, SEEK_SET) )  throw NoInfile();
        }
#ifdef DEBUGOUT
        err_Out1= fopen_s(&f_Out1, DEBUGOUT1, "wb"); if (err_Out1 != 0) throw NoOutfile();
#endif

        #else
        f_V1= fopen(cParameter.get_V1Name().c_str(), "rb"); if (f_V1 == NULL) throw NoInfile();
        f_V2= fopen(cParameter.get_V2Name().c_str(), "rb"); if (f_V2 == NULL) throw NoInfile();
        f_BindD= fopen(cParameter.getBind_DName().c_str(), "rb"); if (f_BindD == NULL) throw NoInfile();
        f_BindR= fopen(cParameter.getBind_RName().c_str(), "wb"); if (f_BindR == NULL) throw NoOutfile();
        if ( VariParam ) {
          f_D1near= fopen(cParameter.getNearFN1().c_str(), "rb"); if (f_D1near == NULL) throw NoInfile();
          f_D1far= fopen(cParameter.getFarFN1().c_str(), "rb"); if (f_D1far == NULL) throw NoInfile();
          f_D1trans= fopen(cParameter.getTransFN1().c_str(), "rb"); if (f_D1trans == NULL) throw NoInfile();
          f_D2near= fopen(cParameter.getNearFN2().c_str(), "rb"); if (f_D2near == NULL) throw NoInfile();
          f_D2far= fopen(cParameter.getFarFN2().c_str(), "rb"); if (f_D2far == NULL) throw NoInfile();
          f_D2trans= fopen(cParameter.getTransFN2().c_str(), "rb"); if (f_D2trans == NULL) throw NoInfile();
          f_V1near= fopen(cParameter.getNearFNV1().c_str(), "rb"); if (f_V1near == NULL) throw NoInfile();
          f_V1far= fopen(cParameter.getFarFNV1().c_str(), "rb"); if (f_V1far == NULL) throw NoInfile();
          f_V1trans= fopen(cParameter.getTransFNV1().c_str(), "rb"); if (f_V1trans == NULL) throw NoInfile();
          f_A1near= fopen(cParameter.getNearFNA1().c_str(), "rb"); if (f_A1near == NULL) throw NoInfile();
          f_A1far= fopen(cParameter.getFarFNA1().c_str(), "rb"); if (f_A1far == NULL) throw NoInfile();
          f_A1trans= fopen(cParameter.getTransFNA1().c_str(), "rb"); if (f_A1trans == NULL) throw NoInfile();
          f_A2near= fopen(cParameter.getNearFNA2().c_str(), "rb"); if (f_A2near == NULL) throw NoInfile();
          f_A2far= fopen(cParameter.getFarFNA2().c_str(), "rb"); if (f_A2far == NULL) throw NoInfile();
          f_A2trans= fopen(cParameter.getTransFNA2().c_str(), "rb"); if (f_A2trans == NULL) throw NoInfile();
          if ( fseek(f_D1near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D1far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D1trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D2near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D2far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_D2trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_V1near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_V1far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_V1trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A1near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A1far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A1trans, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A2near, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A2far, 0, SEEK_SET) )  throw NoInfile();
          if ( fseek(f_A2trans, 0, SEEK_SET) )  throw NoInfile();
        }
        #endif
      } catch (NoInfile) { 
        cout << "GVD: ERR: File Open: Not specify files, or File path/IO error." << endl;
        if (f_V1 != NULL) fclose(f_V1); 
        if (f_V2 != NULL) fclose(f_V2); 
        if ( VariParam ) {
          if (f_D1near != NULL) fclose(f_D1near); 
          if (f_D1far != NULL) fclose(f_D1far); 
          if (f_D1trans != NULL) fclose(f_D1trans); 
          if (f_D2near != NULL) fclose(f_D2near); 
          if (f_D2far != NULL) fclose(f_D2far); 
          if (f_D2trans != NULL) fclose(f_D2trans); 
          if (f_V1near != NULL) fclose(f_V1near); 
          if (f_V1far != NULL) fclose(f_V1far); 
          if (f_V1trans != NULL) fclose(f_V1trans); 
          if (f_A1near != NULL) fclose(f_A1near); 
          if (f_A1far != NULL) fclose(f_A1far); 
          if (f_A1trans != NULL) fclose(f_A1trans); 
          if (f_A2near != NULL) fclose(f_A2near); 
          if (f_A2far != NULL) fclose(f_A2far); 
          if (f_A2trans != NULL) fclose(f_A2trans); 
#ifdef DEBUGOUT
          if (f_Out1 != NULL) fclose(f_Out1); 
#endif
        }
        workR= false;
      } catch (...) {
        cout << "GVD: ERR: File Close ERR." << endl;
        workR= false;
      }
      if ( !workR ) return NO_FILEIO;

      if ( numView == VIEW_3 ) {
        try {
          #ifdef WIN32
          f_V3= _fsopen(cParameter.get_V3Name().c_str(), "rb", _SH_DENYWR); if (f_V3 == NULL) throw NoInfile();
          if ( VariParam ) {
            f_D3near= _fsopen(cParameter.getNearFN3().c_str(), "rb", _SH_DENYWR); if (f_D3near == NULL) throw NoInfile();
            f_D3far= _fsopen(cParameter.getFarFN3().c_str(), "rb", _SH_DENYWR); if (f_D3far == NULL) throw NoInfile();
            f_D3trans= _fsopen(cParameter.getTransFN3().c_str(), "rb", _SH_DENYWR); if (f_D3trans == NULL) throw NoInfile();
            f_V3near= _fsopen(cParameter.getNearFNV2().c_str(), "rb", _SH_DENYWR); if (f_V3near == NULL) throw NoInfile();
            f_V3far= _fsopen(cParameter.getFarFNV2().c_str(), "rb", _SH_DENYWR); if (f_V3far == NULL) throw NoInfile();
            f_V3trans= _fsopen(cParameter.getTransFNV2().c_str(), "rb", _SH_DENYWR); if (f_V3trans == NULL) throw NoInfile();
            f_A3near= _fsopen(cParameter.getNearFNA3().c_str(), "rb", _SH_DENYWR); if (f_A3near == NULL) throw NoInfile();
            f_A3far= _fsopen(cParameter.getFarFNA3().c_str(), "rb", _SH_DENYWR); if (f_A3far == NULL) throw NoInfile();
            f_A3trans= _fsopen(cParameter.getTransFNA3().c_str(), "rb", _SH_DENYWR); if (f_A3trans == NULL) throw NoInfile();
            f_A4near= _fsopen(cParameter.getNearFNA4().c_str(), "rb", _SH_DENYWR); if (f_A4near == NULL) throw NoInfile();
            f_A4far= _fsopen(cParameter.getFarFNA4().c_str(), "rb", _SH_DENYWR); if (f_A4far == NULL) throw NoInfile();
            f_A4trans= _fsopen(cParameter.getTransFNA4().c_str(), "rb", _SH_DENYWR); if (f_A4trans == NULL) throw NoInfile();
            if ( fseek(f_D3near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_D3far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_D3trans, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_V3near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_V3far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_V3trans, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A3near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A3far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A3trans, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A4near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A4far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A4trans, 0, SEEK_SET) )  throw NoInfile();
          }
#ifdef DEBUGOUT
          err_Out2= fopen_s(&f_Out2, DEBUGOUT2, "wb"); if (err_Out2 != 0) throw NoOutfile();
#endif
          #else
          f_V3= fopen(cParameter.get_V3Name().c_str(), "rb"); if (f_V3 == NULL) throw NoInfile();
          if ( VariParam ) {
            f_D3near= fopen(cParameter.getNearFN3().c_str(), "rb"); if (f_D3near == NULL) throw NoInfile();
            f_D3far= fopen(cParameter.getFarFN3().c_str(), "rb"); if (f_D3far == NULL) throw NoInfile();
            f_D3trans= fopen(cParameter.getTransFN3().c_str(), "rb"); if (f_D3trans == NULL) throw NoInfile();
            f_V3near= fopen(cParameter.getNearFNV2().c_str(), "rb"); if (f_V3near == NULL) throw NoInfile();
            f_V3far= fopen(cParameter.getFarFNV2().c_str(), "rb"); if (f_V3far == NULL) throw NoInfile();
            f_V3trans= fopen(cParameter.getTransFNV2().c_str(), "rb"); if (f_V3trans == NULL) throw NoInfile();
            f_A3near= fopen(cParameter.getNearFNA3().c_str(), "rb"); if (f_A3near == NULL) throw NoInfile();
            f_A3far= fopen(cParameter.getFarFNA3().c_str(), "rb"); if (f_A3far == NULL) throw NoInfile();
            f_A3trans= fopen(cParameter.getTransFNA3().c_str(), "rb"); if (f_A3trans == NULL) throw NoInfile();
            f_A4near= fopen(cParameter.getNearFNA4().c_str(), "rb"); if (f_A4near == NULL) throw NoInfile();
            f_A4far= fopen(cParameter.getFarFNA4().c_str(), "rb"); if (f_A4far == NULL) throw NoInfile();
            f_A4trans= fopen(cParameter.getTransFNA4().c_str(), "rb"); if (f_A4trans == NULL) throw NoInfile();
            if ( fseek(f_D3near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_D3far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_D3trans, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_V3near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_V3far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_V3trans, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A3near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A3far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A3trans, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A4near, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A4far, 0, SEEK_SET) )  throw NoInfile();
            if ( fseek(f_A4trans, 0, SEEK_SET) )  throw NoInfile();
          }
          #endif
        } catch (NoInfile) { 
          cout << "GVD: ERR: 3 view files: Not specify files, or File path/IO error." << endl;
          if (f_V3 != NULL) fclose(f_V3); 
          if ( VariParam ) {
            if (f_D3near != NULL) fclose(f_D3near); 
            if (f_D3far != NULL) fclose(f_D3far); 
            if (f_D3trans != NULL) fclose(f_D3trans); 
            if (f_V3near != NULL) fclose(f_V3near); 
            if (f_V3far != NULL) fclose(f_V3far); 
            if (f_V3trans != NULL) fclose(f_V3trans); 
            if (f_A3near != NULL) fclose(f_A3near); 
            if (f_A3far != NULL) fclose(f_A3far); 
            if (f_A3trans != NULL) fclose(f_A3trans); 
            if (f_A4near != NULL) fclose(f_A4near); 
            if (f_A4far != NULL) fclose(f_A4far); 
            if (f_A4trans != NULL) fclose(f_A4trans); 
#ifdef DEBUGOUT
            if (f_Out2 != NULL) fclose(f_Out2); 
#endif
          }
          workR= false;
        } catch (...) {
          cout << "GVD: ERR: 3 view files: File error." << endl;
          workR= false;
        }
        if ( !workR ) return NO_FILEIO;
      }


      if ( VariParam ) {
        #ifdef WIN32
        if ( fscanf_s(f_D1near, "%lf", &near1) != 1 ) return false;
        if ( fscanf_s(f_D1far, "%lf", &far1) != 1 ) return false;
        if ( fscanf_s(f_D1trans, "%lf", &trans1) != 1 ) return false;
        if ( fscanf_s(f_D2near, "%lf", &near2) != 1 ) return false;
        if ( fscanf_s(f_D2far, "%lf", &far2) != 1 ) return false;
        if ( fscanf_s(f_D2trans, "%lf", &trans2) != 1 ) return false;
        if ( fscanf_s(f_V1near, "%lf", &nearV1) != 1 ) return false;
        if ( fscanf_s(f_V1far, "%lf", &farV1) != 1 ) return false;
        if ( fscanf_s(f_V1trans, "%lf", &transV1) != 1 ) return false;
        if ( fscanf_s(f_A1near, "%lf", &nearA1) != 1 ) return false;
        if ( fscanf_s(f_A1far, "%lf", &farA1) != 1 ) return false;
        if ( fscanf_s(f_A1trans, "%lf", &transA1) != 1 ) return false;
        if ( fscanf_s(f_A2near, "%lf", &nearA2) != 1 ) return false;
        if ( fscanf_s(f_A2far, "%lf", &farA2) != 1 ) return false;
        if ( fscanf_s(f_A2trans, "%lf", &transA2) != 1 ) return false;
        if ( fseek(f_D1near, 0, SEEK_SET) )  return false;
        if ( fseek(f_D1far, 0, SEEK_SET) )  return false;
        if ( fseek(f_D1trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_D2near, 0, SEEK_SET) )  return false;
        if ( fseek(f_D2far, 0, SEEK_SET) )  return false;
        if ( fseek(f_D2trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_V1near, 0, SEEK_SET) )  return false;
        if ( fseek(f_V1far, 0, SEEK_SET) )  return false;
        if ( fseek(f_V1trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_A1near, 0, SEEK_SET) )  return false;
        if ( fseek(f_A1far, 0, SEEK_SET) )  return false;
        if ( fseek(f_A1trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_A2near, 0, SEEK_SET) )  return false;
        if ( fseek(f_A2far, 0, SEEK_SET) )  return false;
        if ( fseek(f_A2trans, 0, SEEK_SET) )  return false;
        if ( numView == VIEW_3 ) {
          if ( fscanf_s(f_D3near, "%lf", &near3) != 1 ) return false;
          if ( fscanf_s(f_D3far, "%lf", &far3) != 1 ) return false;
          if ( fscanf_s(f_D3trans, "%lf", &trans3) != 1 ) return false;
          if ( fscanf_s(f_V3near, "%lf", &nearV2) != 1 ) return false;
          if ( fscanf_s(f_V3far, "%lf", &farV2) != 1 ) return false;
          if ( fscanf_s(f_V3trans, "%lf", &transV2) != 1 ) return false;
          if ( fscanf_s(f_A3near, "%lf", &nearA3) != 1 ) return false;
          if ( fscanf_s(f_A3far, "%lf", &farA3) != 1 ) return false;
          if ( fscanf_s(f_A3trans, "%lf", &transA3) != 1 ) return false;
          if ( fscanf_s(f_A4near, "%lf", &nearA4) != 1 ) return false;
          if ( fscanf_s(f_A4far, "%lf", &farA4) != 1 ) return false;
          if ( fscanf_s(f_A4trans, "%lf", &transA4) != 1 ) return false;
          if ( fseek(f_D3near, 0, SEEK_SET) )  return false;
          if ( fseek(f_D3far, 0, SEEK_SET) )  return false;
          if ( fseek(f_D3trans, 0, SEEK_SET) )  return false;
          if ( fseek(f_V3near, 0, SEEK_SET) )  return false;
          if ( fseek(f_V3far, 0, SEEK_SET) )  return false;
          if ( fseek(f_V3trans, 0, SEEK_SET) )  return false;
          if ( fseek(f_A3near, 0, SEEK_SET) )  return false;
          if ( fseek(f_A3far, 0, SEEK_SET) )  return false;
          if ( fseek(f_A3trans, 0, SEEK_SET) )  return false;
          if ( fseek(f_A4near, 0, SEEK_SET) )  return false;
          if ( fseek(f_A4far, 0, SEEK_SET) )  return false;
          if ( fseek(f_A4trans, 0, SEEK_SET) )  return false;
        }
        
        #else
        if ( fscanf(f_D1near, "%lf", &near1) != 1 ) return false;
        if ( fscanf(f_D1far, "%lf", &far1) != 1 ) return false;
        if ( fscanf(f_D1trans, "%lf", &trans1) != 1 ) return false;
        if ( fscanf(f_D2near, "%lf", &near2) != 1 ) return false;
        if ( fscanf(f_D2far, "%lf", &far2) != 1 ) return false;
        if ( fscanf(f_D2trans, "%lf", &trans2) != 1 ) return false;
        if ( fscanf(f_V1near, "%lf", &nearV1) != 1 ) return false;
        if ( fscanf(f_V1far, "%lf", &farV1) != 1 ) return false;
        if ( fscanf(f_V1trans, "%lf", &transV1) != 1 ) return false;
        if ( fscanf(f_A1near, "%lf", &nearA1) != 1 ) return false;
        if ( fscanf(f_A1far, "%lf", &farA1) != 1 ) return false;
        if ( fscanf(f_A1trans, "%lf", &transA1) != 1 ) return false;
        if ( fscanf(f_A2near, "%lf", &nearA2) != 1 ) return false;
        if ( fscanf(f_A2far, "%lf", &farA2) != 1 ) return false;
        if ( fscanf(f_A2trans, "%lf", &transA2) != 1 ) return false;
        if ( fseek(f_D1near, 0, SEEK_SET) )  return false;
        if ( fseek(f_D1far, 0, SEEK_SET) )  return false;
        if ( fseek(f_D1trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_D2near, 0, SEEK_SET) )  return false;
        if ( fseek(f_D2far, 0, SEEK_SET) )  return false;
        if ( fseek(f_D2trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_V1near, 0, SEEK_SET) )  return false;
        if ( fseek(f_V1far, 0, SEEK_SET) )  return false;
        if ( fseek(f_V1trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_A1near, 0, SEEK_SET) )  return false;
        if ( fseek(f_A1far, 0, SEEK_SET) )  return false;
        if ( fseek(f_A1trans, 0, SEEK_SET) )  return false;
        if ( fseek(f_A2near, 0, SEEK_SET) )  return false;
        if ( fseek(f_A2far, 0, SEEK_SET) )  return false;
        if ( fseek(f_A2trans, 0, SEEK_SET) )  return false;
        if ( numView == VIEW_3 ) {
          if ( fscanf(f_D3near, "%lf", &near3) != 1 ) return false;
          if ( fscanf(f_D3far, "%lf", &far3) != 1 ) return false;
          if ( fscanf(f_D3trans, "%lf", &trans3) != 1 ) return false;
          if ( fscanf(f_V3near, "%lf", &nearV2) != 1 ) return false;
          if ( fscanf(f_V3far, "%lf", &farV2) != 1 ) return false;
          if ( fscanf(f_V3trans, "%lf", &transV2) != 1 ) return false;
          if ( fscanf(f_A3near, "%lf", &nearA3) != 1 ) return false;
          if ( fscanf(f_A3far, "%lf", &farA3) != 1 ) return false;
          if ( fscanf(f_A3trans, "%lf", &transA3) != 1 ) return false;
          if ( fscanf(f_A4near, "%lf", &nearA4) != 1 ) return false;
          if ( fscanf(f_A4far, "%lf", &farA4) != 1 ) return false;
          if ( fscanf(f_A4trans, "%lf", &transA4) != 1 ) return false;
          if ( fseek(f_D3near, 0, SEEK_SET) )  return false;
          if ( fseek(f_D3far, 0, SEEK_SET) )  return false;
          if ( fseek(f_D3trans, 0, SEEK_SET) )  return false;
          if ( fseek(f_V3near, 0, SEEK_SET) )  return false;
          if ( fseek(f_V3far, 0, SEEK_SET) )  return false;
          if ( fseek(f_V3trans, 0, SEEK_SET) )  return false;
          if ( fseek(f_A3near, 0, SEEK_SET) )  return false;
          if ( fseek(f_A3far, 0, SEEK_SET) )  return false;
          if ( fseek(f_A3trans, 0, SEEK_SET) )  return false;
          if ( fseek(f_A4near, 0, SEEK_SET) )  return false;
          if ( fseek(f_A4far, 0, SEEK_SET) )  return false;
          if ( fseek(f_A4trans, 0, SEEK_SET) )  return false;
        }
        #endif
      }


      return NORMAL_END;
    }



    void endFile() {
      if ( workR ) {
        fclose(f_V1); fclose(f_BindD); fclose(f_BindR);
        fclose(f_V2);
#ifdef DEBUGOUT
        fclose(f_Out1); 
#endif
        if ( VariParam ) {
          fclose(f_D1near); fclose(f_D1far); fclose(f_D1trans);
          fclose(f_D2near); fclose(f_D2far); fclose(f_D2trans);
          fclose(f_V1near); fclose(f_V1far); fclose(f_V1trans);
          fclose(f_A1near); fclose(f_A1far); fclose(f_A1trans);
          fclose(f_A2near); fclose(f_A2far); fclose(f_A2trans);
        }
        if ( numView == VIEW_3 ) {
          fclose(f_V3);
          if ( VariParam ) {
            fclose(f_D3near); fclose(f_D3far); fclose(f_D3trans);
            fclose(f_V3near); fclose(f_V3far); fclose(f_V3trans);
            fclose(f_A3near); fclose(f_A3far); fclose(f_A3trans);
            fclose(f_A4near); fclose(f_A4far); fclose(f_A4trans);
          }
#ifdef DEBUGOUT
          fclose(f_Out2); 
#endif
        }
      }
    }



    CSubSepMix cSubH;

    int initSubMix() {
      if ( numView == VIEW_2 ) { // for VIEW_2 ///////////////////////////////////////////////////////////////////
        if ( !cSubH.Init(
                  mode, numView, VariParam, width, height, half_width, half_height, 
                  near1, far1, near2, far2, 0, 0,
                  cParameter.getCameraName1().c_str(), 
                  cParameter.getCameraName2().c_str(), 
                  "", 
                  cParameter.getVertualCameraName1().c_str(),
                  "",
                  cParameter.getAddCameraName1().c_str(), 
                  cParameter.getAddCameraName2().c_str(), 
                  "",
                  "",
                  cParameter.getMat_In_1(), cParameter.getMat_Ex_1(), cParameter.getMat_Trans_1(),
                  cParameter.getMat_In_2(), cParameter.getMat_Ex_2(), cParameter.getMat_Trans_2(),
                  cParameter.getMat_In_3(), cParameter.getMat_Ex_3(), cParameter.getMat_Trans_3(),
                  cParameter.getMat_In_V1(), cParameter.getMat_Ex_V1(), cParameter.getMat_Trans_V1(),
                  cParameter.getMat_In_V2(), cParameter.getMat_Ex_V2(), cParameter.getMat_Trans_V2(),
                  cParameter.getMat_In_A1(), cParameter.getMat_Ex_A1(), cParameter.getMat_Trans_A1(),
                  cParameter.getMat_In_A2(), cParameter.getMat_Ex_A2(), cParameter.getMat_Trans_A2(),
                  cParameter.getMat_In_A3(), cParameter.getMat_Ex_A3(), cParameter.getMat_Trans_A3(),
                  cParameter.getMat_In_A4(), cParameter.getMat_Ex_A4(), cParameter.getMat_Trans_A4()
        ) ) { noInitSUBMIX; return NO_INIT_SUBMIX_D; }
      } else { // for VIEW_3 //////////////////////////////////////////////////////////////////
        if ( !cSubH.Init(
                  mode, numView, VariParam, width, height, half_width, height,
                  near1, far1, near2, far2, near3, far3,
                  cParameter.getCameraName1().c_str(), 
                  cParameter.getCameraName2().c_str(), 
                  cParameter.getCameraName3().c_str(), 
                  cParameter.getVertualCameraName1().c_str(),
                  cParameter.getVertualCameraName2().c_str(),
                  cParameter.getAddCameraName1().c_str(), 
                  cParameter.getAddCameraName2().c_str(), 
                  cParameter.getAddCameraName3().c_str(), 
                  cParameter.getAddCameraName4().c_str(), 
                  cParameter.getMat_In_1(), cParameter.getMat_Ex_1(), cParameter.getMat_Trans_1(),
                  cParameter.getMat_In_2(), cParameter.getMat_Ex_2(), cParameter.getMat_Trans_2(),
                  cParameter.getMat_In_3(), cParameter.getMat_Ex_3(), cParameter.getMat_Trans_3(),
                  cParameter.getMat_In_V1(), cParameter.getMat_Ex_V1(), cParameter.getMat_Trans_V1(),
                  cParameter.getMat_In_V2(), cParameter.getMat_Ex_V2(), cParameter.getMat_Trans_V2(),
                  cParameter.getMat_In_A1(), cParameter.getMat_Ex_A1(), cParameter.getMat_Trans_A1(),
                  cParameter.getMat_In_A2(), cParameter.getMat_Ex_A2(), cParameter.getMat_Trans_A2(),
                  cParameter.getMat_In_A3(), cParameter.getMat_Ex_A3(), cParameter.getMat_Trans_A3(),
                  cParameter.getMat_In_A4(), cParameter.getMat_Ex_A4(), cParameter.getMat_Trans_A4()
        ) ) { noInitSUBMIX; return NO_INIT_SUBMIX_D; }
      }
      return NORMAL_END;
    }



    int convOneFrames(unsigned int fnum, unsigned int fnumD) {
      if ( numView == VIEW_2 ) { // for VIEW_2 ///////////////////////////////////////////////////////////////////
        if (workR) {
          if (!cSubH.ReadOneFrame(
                  f_V1, f_V2, NULL, f_BindD, fnum, fnumD,
                  // for VariParam
                  f_D1near, f_D1far, f_D1trans,
                  f_D2near, f_D2far, f_D2trans,
                  NULL, NULL, NULL,
                  f_V1near, f_V1far, f_V1trans,
                  NULL, NULL, NULL,
                  f_A1near, f_A1far, f_A1trans,
                  f_A2near, f_A2far, f_A2trans,
                  NULL, NULL, NULL,
                  NULL, NULL, NULL
          )) return DATA_SET_ERR;
          cout << ".";
#ifndef DEBUGOUT          
          if (!cSubH.DoSubMix(f_BindR)) return DATA_OUT_ERR;
#else
          if (!cSubH.DoSubMix(f_Out1, f_Out2)) return DATA_OUT_ERR;
#endif
          cout << ".";
        }
      } else { // VIEW_3 //////////////////////////////////////////////////////////////////
        if (workR) {
          if (!cSubH.ReadOneFrame(
                  f_V1, f_V2, f_V3, f_BindD, fnum, fnumD,
                  // for VariParam
                  f_D1near, f_D1far, f_D1trans,
                  f_D2near, f_D2far, f_D2trans,
                  f_D3near, f_D3far, f_D3trans,
                  f_V1near, f_V1far, f_V1trans,
                  f_V3near, f_V3far, f_V3trans,
                  f_A1near, f_A1far, f_A1trans,
                  f_A2near, f_A2far, f_A2trans,
                  f_A3near, f_A3far, f_A3trans,
                  f_A4near, f_A4far, f_A4trans
          )) return DATA_SET_ERR;
          cout << ".";
#ifndef DEBUGOUT          
          if (!cSubH.DoSubMix(f_BindR)) return DATA_OUT_ERR;
#else
          if (!cSubH.DoSubMix(f_Out1, f_Out2)) return DATA_OUT_ERR;
#endif
          cout << ".";
        }
      }
      return NORMAL_END;
    }

  }

}
