// View Synthesis Encoder for Residuals with GT-Fly
// Dev Enviroment:
//    Windows 7 64bit with Visual Studio 2010 C++ x64
//    OpenCV 2.4.2 64bit
// Last Update: Feb, 2014.

// Original code: View Synthesis Reference Software (VSRS) 3.5
//======================================created by Ying Chen =====================================
//===============Tampere University of Technology (TUT)/Nokia Research Center (NRC)===============

#ifdef WIN32
#pragma warning(disable : 4996)
#else
#include <stdio.h>
#include <string.h>
#endif

#include "ParameterBase.h"
#include "ParameterEncResidual.h"
#include "SubMixOptions.h"

// Modified by NICT. Dec 2011
CParameterEncResiduals::CParameterEncResiduals()
  : m_Mode ( "" )
  , m_uiSourceWidth ( 0 )
  , m_uiSourceHeight ( 0 )
  , m_uiStartFrame ( 0 ) 
  , m_uiNumberOfFrames ( 0 )
  , m_Offset ( "" ) 

  , m_dNearestDepthValue1 ( 0 )
  , m_dFarthestDepthValue1 ( 0 )
  , m_dNearestDepthValue2 ( 0 )
  , m_dFarthestDepthValue2 ( 0 )
  , m_dNearestDepthValue3 ( 0 )
  , m_dFarthestDepthValue3 ( 0 )

  , m_CameraName1 ( "" )
  , m_CameraName2 ( "" )
  , m_CameraName3 ( "" )
  , m_VertualCameraName1 ( "" )
  , m_VertualCameraName2 ( "" )
  , m_AddCameraName1 ( "" )
  , m_AddCameraName2 ( "" )
  , m_AddCameraName3 ( "" )
  , m_AddCameraName4 ( "" )

  , m_CameraParameterFileName ( "" )

  , m_NearFN1 ( "" )
  , m_FarFN1 ( "" )
  , m_TransFN1 ( "" )
  , m_NearFN2 ( "" )
  , m_FarFN2 ( "" )
  , m_TransFN2 ( "" )
  , m_NearFN3 ( "" )
  , m_FarFN3 ( "" )
  , m_TransFN3 ( "" )
  , m_NearFNV1 ( "" )
  , m_FarFNV1 ( "" )
  , m_TransFNV1 ( "" )
  , m_NearFNV2 ( "" )
  , m_FarFNV2 ( "" )
  , m_TransFNV2 ( "" )
  , m_NearFNA1 ( "" )
  , m_FarFNA1 ( "" )
  , m_TransFNA1 ( "" )
  , m_NearFNA2 ( "" )
  , m_FarFNA2 ( "" )
  , m_TransFNA2 ( "" )
  , m_NearFNA3 ( "" )
  , m_FarFNA3 ( "" )
  , m_TransFNA3 ( "" )
  , m_NearFNA4 ( "" )
  , m_FarFNA4 ( "" )
  , m_TransFNA4 ( "" )

  , m_V1Name ( "" )
  , m_V2Name ( "" )
  , m_V3Name ( "" )
  , m_Bind_DName ( "" )
  , m_Bind_RName ( "" )
{
}



// Original
CParameterEncResiduals::~CParameterEncResiduals() {
  release();
}



// Small modified by NICT. Dec 2011
Int CParameterEncResiduals::Init( Int argc, Char**  argv ) {
  setup();

  if ( argc < 2 ) {
    xPrintUsage();
    release();
    return -1;
  }
  
  std::string cFilename= argv[1] ; 
  if ( xReadFromFile( cFilename ) != 1 ) {
    xPrintUsage();
    release();
    return -1;
  }

  if ( xReadFromCommandLine(argc, argv) != 1 ) {
    xPrintUsage();
    release();
    return -1;
  }
  xPrintParam();
  release();

  if ( !xReadCameraParameters() ) return -1;

  return 1;  
}



// New impliment by NICT. Dec 2011
void CParameterEncResiduals::xPrintUsage() {
  using namespace std;
  
  cout << endl << "GVD: ERR: Config file name required." << endl;
  cout <<         "  Config files parameter set:" << endl;

  unsigned int uiParLnCount= 0;
  while ( m_pCfgLines[uiParLnCount] != NULL ) {
    cout << "    " << m_pCfgLines[uiParLnCount]->getTag() << endl;
    uiParLnCount++;
  }
  cout << endl << "      Mode: ""QUARTER / QUATER_L"" or ""QUARTER_ODD / QUARTER_ODD_L / FULL_ODD_L""." << endl;
  cout <<         "        QUARTER / QUATER_L mode is for 2 View setting." << endl;
  cout <<         "        QUARTER_ODD / QUARTER_ODD_L / FULL_ODD_L mode is for 3 View setting." << endl;
  cout <<         "        Without postfix ""_L"" mode is for Half hight/width output." << endl;
  cout <<         "        With postfix ""_L"" mode is for Half hight/width depth map " << endl;
  cout <<         "                                               with full size output." << endl << endl;
  cout <<         "      GVD standard: 2views = QUATER_L, 3views(ATM) = QUARTER_ODD_L, 3views(HTM) = FULL_ODD_L." << endl;
  cout <<         "      File/Position ID 1,2,3.. is LEFT:1 to RIGHT:3." << endl;
}



// Modified by NICT. Dec 2011
unsigned int CParameterEncResiduals::setup() {
  unsigned int uiParLnCount= 0;

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Mode", &m_Mode, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "SourceWidth", &m_uiSourceWidth, 1280 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "SourceHeight", &m_uiSourceHeight, 960 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "TotalNumberOfFrames", &m_uiNumberOfFrames, 1 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineUInt( "StartFrame", &m_uiStartFrame, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Offset", &m_Offset, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue1", &m_dNearestDepthValue1, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue1", &m_dFarthestDepthValue1, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue2", &m_dNearestDepthValue2, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue2", &m_dFarthestDepthValue2, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "NearestDepthValue3", &m_dNearestDepthValue3, 0 );
  m_pCfgLines[uiParLnCount++]= new ConfigLineDbl( "FarthestDepthValue3", &m_dFarthestDepthValue3, 0 );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName1", &m_CameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName2", &m_CameraName2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraName3", &m_CameraName3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "VertualCameraName1", &m_VertualCameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "VertualCameraName2", &m_VertualCameraName2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "AddCameraName1", &m_AddCameraName1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "AddCameraName2", &m_AddCameraName2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "AddCameraName3", &m_AddCameraName3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "AddCameraName4", &m_AddCameraName4, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "CameraParameterFile", &m_CameraParameterFileName, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Near1", &m_NearFN1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Far1", &m_FarFN1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Trans1", &m_TransFN1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Near2", &m_NearFN2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Far2", &m_FarFN2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Trans2", &m_TransFN2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Near3", &m_NearFN3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Far3", &m_FarFN3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "Trans3", &m_TransFN3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearV1", &m_NearFNV1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarV1", &m_FarFNV1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransV1", &m_TransFNV1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearV2", &m_NearFNV2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarV2", &m_FarFNV2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransV2", &m_TransFNV2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearA1", &m_NearFNA1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarA1", &m_FarFNA1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransA1", &m_TransFNA1, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearA2", &m_NearFNA2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarA2", &m_FarFNA2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransA2", &m_TransFNA2, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearA3", &m_NearFNA3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarA3", &m_FarFNA3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransA3", &m_TransFNA3, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "NearA4", &m_NearFNA4, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "FarA4", &m_FarFNA4, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "TransA4", &m_TransFNA4, "" );

  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "View1", &m_V1Name, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "View2", &m_V2Name, "" ); // Mar12,2012 for avgMaskColor
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "View3", &m_V3Name, "" ); // Dec28,2011 ex.View2
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "DepthBind", &m_Bind_DName, "" );
  m_pCfgLines[uiParLnCount++]= new ConfigLineStr( "ResidualsBind", &m_Bind_RName, "" );

  m_pCfgLines[uiParLnCount] = NULL;

  return uiParLnCount;
}



// Small modified by NICT. Dec 2011
bool CParameterEncResiduals::xReadCameraParameters() {
  FILE *f_CPF;
  class NoInfile {};
  try {
    #ifdef WIN32
    f_CPF= _fsopen(m_CameraParameterFileName.c_str(), "rt", _SH_DENYWR); if (f_CPF == NULL) throw NoInfile(); // rb
    #else
    f_CPF= fopen(m_CameraParameterFileName.c_str(), "rt"); if (f_CPF == NULL) throw NoInfile(); // rb
    #endif
  } catch (NoInfile) { 
    std::cout << "GVD: ERR: Camera parameter file: Not specify files, or File path/IO error." << std::endl;
  } catch (...) {
    std::cout << "GVD: ERR: Camera parameter file: File IO error." << std::endl;
  }
  
  bool is3view= ( m_Mode.compare("QUARTER") != 0 && m_Mode.compare("QUARTER_L") != 0 );

  const char* cameraId[CAMERAS];
  try {
    cameraId[0]= m_CameraName1.c_str();
    cameraId[1]= m_CameraName2.c_str();
    cameraId[2]= "";
    cameraId[3]= m_VertualCameraName1.c_str();
    cameraId[4]= "";
    cameraId[5]= m_AddCameraName1.c_str();
    cameraId[6]= m_AddCameraName2.c_str();
    cameraId[7]= "";
    cameraId[8]= "";
    if ( is3view ) {
      cameraId[2]= m_CameraName3.c_str();
      cameraId[4]= m_VertualCameraName2.c_str();
      cameraId[7]= m_AddCameraName3.c_str();
      cameraId[8]= m_AddCameraName4.c_str();
    }
  } catch (...) {
    std::cout << "VSencR: ERR: Camera parameter file: File IO error." << std::endl;
  }

  char id[255];
  double gomi[2];
  bool found;
  int read= 0;
  for (unsigned int i= 0; i < CAMERAS; i++) {
    if ( !is3view && (i == 2 || i == 4 || i == 7 || i == 8) ) continue; 
    if ( fseek(f_CPF, 0, SEEK_SET) ) return 0;

    found= false;
    while( fscanf(f_CPF, "%s", id) == 1 ) {
      read= 0;
      if( strcmp(cameraId[i], id) == 0 ) {
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[0][0], &m_camParam[i].m_fIntrinsicMatrix[0][1], &m_camParam[i].m_fIntrinsicMatrix[0][2]);
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[1][0], &m_camParam[i].m_fIntrinsicMatrix[1][1], &m_camParam[i].m_fIntrinsicMatrix[1][2]);
        read += fscanf(f_CPF, "%lf %lf %lf", &m_camParam[i].m_fIntrinsicMatrix[2][0], &m_camParam[i].m_fIntrinsicMatrix[2][1], &m_camParam[i].m_fIntrinsicMatrix[2][2]);
        read += fscanf(f_CPF, "%lf %lf", &gomi[0], &gomi[1]);
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[0][0], &m_camParam[i].m_fExtrinsicMatrix[0][1], &m_camParam[i].m_fExtrinsicMatrix[0][2], &m_camParam[i].m_fTranslationVector[0] );
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[1][0], &m_camParam[i].m_fExtrinsicMatrix[1][1], &m_camParam[i].m_fExtrinsicMatrix[1][2], &m_camParam[i].m_fTranslationVector[1] );
        read += fscanf(f_CPF, "%lf %lf %lf %lf", &m_camParam[i].m_fExtrinsicMatrix[2][0], &m_camParam[i].m_fExtrinsicMatrix[2][1], &m_camParam[i].m_fExtrinsicMatrix[2][2], &m_camParam[i].m_fTranslationVector[2] );
        if(read!=23) return false;
        found= true;
      }
    }
    
    if (!found) {
      std::cout << "GVD: ERR: Camera ID, not found in the camera parameter file: " << cameraId[i] << std::endl;
      return false;
    }
  }

  return true;
}

